@@ -1,4 +1,4 @@
-/* Copyright (c) 1998-2009, 2011 Thorsten Kukuk
+/* Copyright (c) 1998-2009, 2011, 2012, 2013 Thorsten Kukuk
    This file is part of ypbind-mt.
    Author: Thorsten Kukuk <kukuk@suse.de>
 
@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public
    License along with ypbind-mt; see the file COPYING.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA. */
+   write to the Free Software Foundation, Inc., 51 Franklin Street - Suite 500,
+   Boston, MA 02110-1335, USA. */
 
 #define _GNU_SOURCE
 
@@ -108,11 +108,37 @@ static pthread_mutex_t search_lock = PTHREAD_MUTEX_INITIALIZER;
 static void do_broadcast (struct binding *list);
 static int ping_all (struct binding *list);
 
-static void
-remove_bindingfile (const char *domain_name)
+/* We have localhost defined in one of the domains.
+ * If so, we don't need to be connected to outer network. */
+void
+check_localhost()
 {
+  int i, s;
+  localhost_used = 0;
+  for (i = 0; i < max_domains; ++i)
+    {
+      for (s = 0; s < _MAXSERVER; ++s)
+        {
+	  if (domainlist[i].server[s].host == NULL)
+	    break;
+          if (strncmp(inet_ntoa(domainlist[i].server[s].addr), "127", 3) == 0)
+            {
+       	      localhost_used = 1;
+      	      return;
+            }
+        }
+    }
+}
+
+static void
+remove_bindingfile (struct binding *entry)
+{
+  const char *domain_name = entry->domain;
   char path[strlen (BINDINGDIR) + strlen (domain_name) + 10];
 
+  /* We want to re-create binding files after binding is OK again */
+  entry->last.host = NULL;
+
   sprintf (path, "%s/%s.1", BINDINGDIR, domain_name);
   unlink (path);
   sprintf (path, "%s/%s.2", BINDINGDIR, domain_name);
@@ -216,6 +242,7 @@ update_bindingfile (struct binding *entry)
     }
   else
     log_msg (LOG_ERR, "open(%s): %s", path2, strerror (errno));
+  check_localhost();
 }
 
 /* this is called from the RPC thread (ypset). */
@@ -375,7 +402,7 @@ clear_server (void)
 	{
 	  if (domainlist[i].active != -1)
 	    {
-	      remove_bindingfile (domainlist[i].domain);
+	      remove_bindingfile (&domainlist[i]);
 	      for (j = 0; j < _MAXSERVER; ++j)
 		{
 		  if (domainlist[i].server[j].host != NULL)
@@ -558,6 +585,7 @@ add_server (const char *domain, const char *host, int check_syntax)
 	 If there is none, use the first one. */
       memcpy (&entry->server[active].addr, hent->h_addr_list[0],
 	      hent->h_length);
+      check_localhost();
       res = 1;
     }
 
@@ -629,11 +657,9 @@ eachresult (bool_t *out, struct sockaddr_in *addr)
           return 0;
         }
 
-      memcpy(&(in_use->server[0].addr), &addr->sin_addr,
-	     sizeof (struct in_addr));
-      memcpy(&(in_use->server[0].port), &addr->sin_port,
-	     sizeof (unsigned short int));
-      memcpy (&(in_use->sin), &addr, sizeof (struct sockaddr_in));
+      in_use->server[0].addr = addr->sin_addr;
+      in_use->server[0].port = addr->sin_port;
+      in_use->sin = *addr;
 
       in_use->active = 0;
 
@@ -686,7 +712,7 @@ do_broadcast (struct binding *list)
 
   if (status != RPC_SUCCESS)
     {
-      remove_bindingfile (domain);
+      remove_bindingfile (list);
       log_msg (LOG_ERR, "broadcast: %s.", clnt_sperrno (status));
     }
   else
@@ -830,7 +856,7 @@ ping_all (struct binding *list)
       if (s_in.sin_port == 0)
 	{
 	  if (verbose_flag && list->active == i)
-		log_msg (LOG_NOTICE, "NIS server '%s' not repsonding "
+		log_msg (LOG_NOTICE, "NIS server '%s' not responding "
 		    "for domain '%s'", list->server[i].host, list->domain);
 
 	  if (debug_flag)
@@ -950,7 +976,7 @@ ping_all (struct binding *list)
   free (pings);
 
   if (!found)
-    remove_bindingfile (list->domain);
+    remove_bindingfile (list);
 
   return found;
 }
@@ -1062,7 +1088,7 @@ ping_all (struct binding *list)
       ++i;
       if (i == _MAXSERVER)
         {
-          remove_bindingfile(list->domain);
+          remove_bindingfile(list);
           return 0;
         }
     }
@@ -1103,7 +1129,7 @@ test_bindings (void *param __attribute__ ((unused)))
   int lastcheck = 0;
 
 #ifdef USE_DBUS_NM
-  if (is_online)
+  if (is_online || localhost_used)
 #endif
     do_binding ();
 
@@ -1119,11 +1145,11 @@ test_bindings (void *param __attribute__ ((unused)))
 	pthread_exit (&success);
 
       lastcheck += ping_interval;
-      if (lastcheck >= 900) /* 900 = 15min. */
+      if (lastcheck >= rebind_interval) /* default 900 = 15min. */
 	lastcheck = 0;
 
 #if USE_DBUS_NM
-      if (is_online)
+      if (is_online || localhost_used)
 
 #endif
 	lastcheck = test_bindings_once (lastcheck, NULL);
